@@ -8,6 +8,7 @@
  * Copyright (C) 1996 Paul Mackerras.
  */
 #include <linux/config.h>
+#include <linux/module.h>
 #include <linux/kernel.h>
 #include <linux/delay.h>
 #include <linux/types.h>
@@ -28,6 +29,11 @@
 #include <asm/hydra.h>
 #include <asm/processor.h>
 #include <asm/spinlock.h>
+#include <asm/feature.h>
+#ifdef CONFIG_PMAC_PBOOK
+#include <asm/adb.h>
+#include <asm/pmu.h>
+#endif
 
 #include "scsi.h"
 #include "hosts.h"
@@ -39,6 +45,8 @@
  * - retry arbitration if lost (unless higher levels do this for us)
  */
 
+#define MESH_NEW_STYLE_EH
+
 #if 1
 #undef KERN_DEBUG
 #define KERN_DEBUG KERN_WARNING
@@ -150,10 +158,14 @@ struct mesh_state {
 	u8	msgout[16];
 	struct dbdma_cmd *dma_cmds;	/* space for dbdma commands, aligned */
 	int	clk_freq;
+	struct mesh_target tgts[8];
+	struct device_node *dnode;
+	unsigned char *mio_base;
+#ifndef MESH_NEW_STYLE_EH
 	Scsi_Cmnd *completed_q;
 	Scsi_Cmnd *completed_qtail;
-	struct mesh_target tgts[8];
 	struct tq_struct tqueue;
+#endif
 #ifdef MESH_DBG
 	int	log_ix;
 	int	n_log;
@@ -161,6 +173,14 @@ struct mesh_state {
 #endif
 };
 
+#ifdef CONFIG_PMAC_PBOOK
+static int mesh_notify_sleep(struct pmu_sleep_notifier *self, int when);
+static struct pmu_sleep_notifier mesh_sleep_notifier = {
+	mesh_notify_sleep,
+	SLEEP_LEVEL_BLOCK,
+};
+#endif
+
 #ifdef MESH_DBG
 
 static void dlog(struct mesh_state *ms, char *fmt, int a);
@@ -185,7 +205,9 @@ static int mesh_notify_reboot(struct notifier_block *, unsigned long, void *);
 static void mesh_dump_regs(struct mesh_state *);
 static void mesh_start(struct mesh_state *);
 static void mesh_start_cmd(struct mesh_state *, Scsi_Cmnd *);
+#ifndef MESH_NEW_STYLE_EH
 static void finish_cmds(void *);
+#endif
 static void add_sdtr_msg(struct mesh_state *);
 static void set_sdtr(struct mesh_state *, int, int);
 static void start_phase(struct mesh_state *);
@@ -206,6 +228,7 @@ static void set_dma_cmds(struct mesh_state *, Scsi_Cmnd *);
 static void halt_dma(struct mesh_state *);
 static int data_goes_out(Scsi_Cmnd *);
 static void do_abort(struct mesh_state *ms);
+static void set_mesh_power(struct mesh_state *ms, int state);
 
 static struct notifier_block mesh_notifier = {
 	mesh_notify_reboot,
@@ -222,6 +245,13 @@ mesh_detect(Scsi_Host_Template *tp)
 	struct Scsi_Host *mesh_host;
 	void *dma_cmd_space;
 
+	if (_machine == _MACH_Pmac) {
+	    use_active_neg = (find_devices("mac-io") ? 0 : SEQ_ACTIVE_NEG);
+	} else {
+	    /* CHRP mac-io */
+	    use_active_neg = SEQ_ACTIVE_NEG;
+	}
+
 	nmeshes = 0;
 	prev_statep = &all_meshes;
 	/*
@@ -233,6 +263,7 @@ mesh_detect(Scsi_Host_Template *tp)
 	if (mesh == 0)
 		mesh = find_compatible_devices("scsi", "chrp,mesh0");
 	for (; mesh != 0; mesh = mesh->next) {
+		struct device_node *mio;
 		if (mesh->n_addrs != 2 || mesh->n_intrs != 2) {
 			printk(KERN_ERR "mesh: expected 2 addrs and 2 intrs"
 			       " (got %d,%d)", mesh->n_addrs, mesh->n_intrs);
@@ -244,13 +275,16 @@ mesh_detect(Scsi_Host_Template *tp)
 			continue;
 		}
 		mesh_host->unique_id = nmeshes;
+#ifndef MODULE
 		note_scsi_host(mesh, mesh_host);
+#endif
 
 		ms = (struct mesh_state *) mesh_host->hostdata;
 		if (ms == 0)
 			panic("no mesh state");
 		memset(ms, 0, sizeof(*ms));
 		ms->host = mesh_host;
+		ms->dnode = mesh;
 		ms->mesh = (volatile struct mesh_regs *)
 			ioremap(mesh->addrs[0].address, 0x1000);
 		ms->dma = (volatile struct dbdma_regs *)
@@ -274,10 +308,10 @@ mesh_detect(Scsi_Host_Template *tp)
 			ms->tgts[tgt].sync_params = ASYNC_PARAMS;
 			ms->tgts[tgt].current_req = 0;
 		}
-
+#ifndef MESH_NEW_STYLE_EH
 		ms->tqueue.routine = finish_cmds;
 		ms->tqueue.data = ms;
-
+#endif
 		*prev_statep = ms;
 		prev_statep = &ms->next;
 
@@ -294,6 +328,16 @@ mesh_detect(Scsi_Host_Template *tp)
 		if (mesh_sync_period < minper)
 			mesh_sync_period = minper;
 
+		ms->mio_base = 0;
+		for (mio = ms->dnode->parent; mio; mio = mio->parent)
+			if (strcmp(mio->name, "mac-io") == 0
+			    && mio->n_addrs > 0)
+				break;
+		if (mio)
+			ms->mio_base = (unsigned char *)
+				ioremap(mio->addrs[0].address, 0x40);
+
+		set_mesh_power(ms, 1);
 		mesh_init(ms);
 
 		if (request_irq(ms->meshintr, do_mesh_interrupt, 0, "MESH", ms)) {
@@ -302,18 +346,75 @@ mesh_detect(Scsi_Host_Template *tp)
 
 		++nmeshes;
 	}
-	if (_machine == _MACH_Pmac) {
-	    use_active_neg = (find_devices("mac-io") ? 0 : SEQ_ACTIVE_NEG);
-	    if (nmeshes > 0)
-		register_reboot_notifier(&mesh_notifier);
-	} else {
-	    /* CHRP mac-io */
-	    use_active_neg = SEQ_ACTIVE_NEG;
-	}
+
+	if ((_machine == _MACH_Pmac) && (nmeshes > 0)) {
+#ifdef CONFIG_PMAC_PBOOK
+		pmu_register_sleep_notifier(&mesh_sleep_notifier);
+#endif /* CONFIG_PMAC_PBOOK */
+ 		register_reboot_notifier(&mesh_notifier);
+	}		
 
 	return nmeshes;
 }
 
+static void
+set_mesh_power(struct mesh_state *ms, int state)
+{
+	if (_machine != _MACH_Pmac || ms->mio_base == 0)
+		return;
+		
+	if (state) {
+		feature_set(ms->dnode, FEATURE_MESH_enable);
+		/* This seems to enable the termination power. strangely
+		   this doesn't fully agree with OF, but with MacOS */
+		if (ms->mio_base)
+			out_8(ms->mio_base + 0x36, 0x70);
+		mdelay(200);
+	} else {
+		feature_clear(ms->dnode, FEATURE_MESH_enable);
+		if (ms->mio_base)
+			out_8(ms->mio_base + 0x36, 0x34);
+		mdelay(10);
+	}
+}			
+
+#ifdef CONFIG_PMAC_PBOOK
+/*
+ * notify clients before sleep and reset bus afterwards
+ */
+int
+mesh_notify_sleep(struct pmu_sleep_notifier *self, int when)
+{
+	struct mesh_state *ms;
+	
+	switch (when) {
+	case PBOOK_SLEEP_REQUEST:
+		/* XXX We should wait for current transactions and queue
+		 * new ones that would be posted beyond this point 
+		 */ 
+		break;
+	case PBOOK_SLEEP_REJECT:
+		break;
+		
+	case PBOOK_SLEEP_NOW:
+		for (ms = all_meshes; ms != 0; ms = ms->next) {
+			disable_irq(ms->meshintr);
+			set_mesh_power(ms, 0);
+		}
+		break;
+	case PBOOK_WAKE:
+		for (ms = all_meshes; ms != 0; ms = ms->next) {
+			set_mesh_power(ms, 1);
+			mesh_init(ms);
+			enable_irq(ms->meshintr);
+		}
+		break;
+	}
+	return PBOOK_SLEEP_OK;
+}
+#endif /* CONFIG_PMAC_PBOOK */
+ 
+
 int
 mesh_queue(Scsi_Cmnd *cmd, void (*done)(Scsi_Cmnd *))
 {
@@ -422,7 +523,9 @@ mesh_reset(Scsi_Cmnd *cmd, unsigned how)
 	{
 		handle_reset(ms);
 		restore_flags(flags);
+#ifndef MESH_NEW_STYLE_EH
 		finish_cmds(ms);
+#endif
 		ret |= SCSI_RESET_SUCCESS;
 	}
 	return ret;
@@ -467,12 +570,27 @@ mesh_init(struct mesh_state *ms)
 	volatile struct mesh_regs *mr = ms->mesh;
 	volatile struct dbdma_regs *md = ms->dma;
 
-	out_8(&mr->interrupt, 0xff);	/* clear all interrupt bits */
+	udelay(100);
+
+	out_le32(&md->control, (RUN|PAUSE|FLUSH|WAKE) << 16);	/* stop dma */
+	out_8(&mr->exception, 0xff);	/* clear all exception bits */
+	out_8(&mr->error, 0xff);	/* clear all error bits */
+	out_8(&mr->sequence, SEQ_RESETMESH);
+	udelay(10);
 	out_8(&mr->intr_mask, INT_ERROR | INT_EXCEPTION | INT_CMDDONE);
 	out_8(&mr->source_id, ms->host->this_id);
 	out_8(&mr->sel_timeout, 25);	/* 250ms */
-	out_8(&mr->sync_params, ASYNC_PARAMS);	/* asynchronous initially */
-	out_le32(&md->control, (RUN|PAUSE|FLUSH|WAKE) << 16);
+	out_8(&mr->sync_params, ASYNC_PARAMS);
+
+	out_8(&mr->bus_status1, BS1_RST);	/* assert RST */
+	udelay(30);			/* leave it on for >= 25us */
+	out_8(&mr->bus_status1, 0);	/* negate RST */
+
+	out_8(&mr->sequence, SEQ_FLUSHFIFO);
+	udelay(1);
+	out_8(&mr->sync_params, ASYNC_PARAMS);
+	out_8(&mr->sequence, SEQ_ENBRESEL);
+	out_8(&mr->interrupt, 0xff);	/* clear all interrupt bits */
 }
 
 /*
@@ -656,6 +774,7 @@ mesh_start_cmd(struct mesh_state *ms, Scsi_Cmnd *cmd)
 	}
 }
 
+#ifndef MESH_NEW_STYLE_EH
 static void
 finish_cmds(void *data)
 {
@@ -664,18 +783,18 @@ finish_cmds(void *data)
 	unsigned long flags;
 
 	for (;;) {
-		save_flags(flags);
-		cli();
+		spin_lock_irqsave(&io_request_lock, flags);
 		cmd = ms->completed_q;
 		if (cmd == NULL) {
-			restore_flags(flags);
+			spin_unlock_irqrestore(&io_request_lock, flags);
 			break;
 		}
 		ms->completed_q = (Scsi_Cmnd *) cmd->host_scribble;
-		restore_flags(flags);
 		(*cmd->scsi_done)(cmd);
+		spin_unlock_irqrestore(&io_request_lock, flags);
 	}
 }
+#endif /* MESH_NEW_STYLE_EH */
 
 static inline void
 add_sdtr_msg(struct mesh_state *ms)
@@ -741,8 +860,8 @@ start_phase(struct mesh_state *ms)
 	Scsi_Cmnd *cmd = ms->current_req;
 	struct mesh_target *tp = &ms->tgts[ms->conn_tgt];
 
-	dlog(ms, "start_phase err/exc/fc/seq = %.8x",
-	     MKWORD(mr->error, mr->exception, mr->fifo_count, mr->sequence));
+	dlog(ms, "start_phase nmo/exc/fc/seq = %.8x",
+	     MKWORD(ms->n_msgout, mr->exception, mr->fifo_count, mr->sequence));
 	out_8(&mr->interrupt, INT_ERROR | INT_EXCEPTION | INT_CMDDONE);
 	seq = use_active_neg + (ms->n_msgout? SEQ_ATN: 0);
 	switch (ms->msgphase) {
@@ -1036,6 +1155,7 @@ cmd_complete(struct mesh_state *ms)
 			t = 230;		/* wait up to 230us */
 			while ((mr->bus_status0 & BS0_REQ) == 0) {
 				if (--t < 0) {
+					dlog(ms, "impatient for req", ms->n_msgout);
 					ms->msgphase = msg_none;
 					break;
 				}
@@ -1325,11 +1445,11 @@ handle_reset(struct mesh_state *ms)
 static void
 do_mesh_interrupt(int irq, void *dev_id, struct pt_regs *ptregs)
 {
-	/*unsigned long flags;*/
+	unsigned long flags;
 
-	/*spin_lock_irqsave(&io_request_lock, flags);*/
+	spin_lock_irqsave(&io_request_lock, flags);
 	mesh_interrupt(irq, dev_id, ptregs);
-	/*spin_unlock_irqrestore(&io_request_lock, flags);*/
+	spin_unlock_irqrestore(&io_request_lock, flags);
 }
 
 static void handle_error(struct mesh_state *ms)
@@ -1622,6 +1742,9 @@ mesh_done(struct mesh_state *ms, int start_next)
 static void
 mesh_completed(struct mesh_state *ms, Scsi_Cmnd *cmd)
 {
+#ifdef MESH_NEW_STYLE_EH
+	(*cmd->scsi_done)(cmd);
+#else
 	if (ms->completed_q == NULL)
 		ms->completed_q = cmd;
 	else
@@ -1630,6 +1753,7 @@ mesh_completed(struct mesh_state *ms, Scsi_Cmnd *cmd)
 	cmd->host_scribble = NULL;
 	queue_task(&ms->tqueue, &tq_immediate);
 	mark_bh(IMMEDIATE_BH);
+#endif /* MESH_NEW_STYLE_EH */
 }
 
 /*
@@ -1788,6 +1912,12 @@ data_goes_out(Scsi_Cmnd *cmd)
 	}
 }
 
+#ifdef MODULE
+Scsi_Host_Template driver_template = SCSI_MESH;
+
+#include "scsi_module.c"
+#endif /* MODULE */
+
 #ifdef MESH_DBG
 static inline u32 readtb(void)
 {
